"""font.py

Part of the Tundra project.
(C) 2006 Rebuild Software
http://rebui.ld.net.au

This file handles the in-game bitmapped text, as well as custom widget classes.
"""
import globals as g

import sprite

try:
    import pygame
    from pygame.locals import *
except ImportError:
    raise ImportError, "You don't have Pygame installed."
try:
    assert pygame.image.get_extended()
except AssertionError:
    raise ImportError, "Your SDL_image does not support PNG images!"

class Obj: pass

import os

from pgu import gui
from pgu.gui import button, layout, surface, slider

class Minimap(slider._slider):
    def __init__(self, game, **params):
        params.setdefault('cls','minimap')
        self.game = game
        self.surface = pygame.Surface([0,0])
        self.minimapBg = self.surface
        self.dots = sprite.SpriteGroup()
        self.dot = pygame.Surface([3,3],SWSURFACE|SRCCOLORKEY).convert()
        self.dot.fill((0,0,0))
        self.dot.set_colorkey((0,0,0))
        pygame.draw.ellipse(self.dot, (1,1,1), self.dot.get_rect())
        self.dot = self.dot.convert()

        slider._slider.__init__(self,0,slider._SLIDER_HORIZONTAL,0,g.MAX_LEVEL_WIDTH-(g.SCREEN_WIDTH/2), size=(g.SCREEN_WIDTH/2)*g.MINIMAP_WIDTH/g.MAX_LEVEL_WIDTH,width=g.MINIMAP_WIDTH,height=g.MINIMAP_HEIGHT,name='minimap',**params)

    def setMap(self, surface):
        self.surface = surface
        self.updateMap()

    def updateMap(self):
        self.minimapBg = pygame.transform.scale(self.surface, (self.surface.get_height()*(g.MINIMAP_WIDTH-12)/(g.MINIMAP_HEIGHT-12), self.surface.get_height() ) )
        self.minimapBg = pygame.transform.rotozoom(self.minimapBg,0,1.0*(g.MINIMAP_WIDTH-12)/self.minimapBg.get_width())
        
    def paint(self,s):
        s.blit(self.minimapBg, (6,6))
        slider._slider.paint(self,s)
        
    def event(self,e):
        slider._slider.event(self,e)
        self.game.levelRect.left = self.value
        self.game.repaintBg = True

class Icon(button._button):
    def __init__(self,surface,**params):
        button._button.__init__(self,**params)
        self.style.image = surface
        self.style.width = surface.get_width()
        self.style.height = surface.get_height()
        
        self.state = 0
    
    def paint(self,s):
        s.blit(self.style.image,(0,0))

class PixmapButton(button._button):
    """A button, buttons can be clicked, they are usually used to set up callbacks.
    
    <pre>Button(value=None)</pre>
    
    <dl>
    <dt>value<dd>either a widget or a string
    </dl>
    
    <strong>Example</strong>
    <code>
    w = gui.Button(image, "ARSE")
    w.connect(gui.CLICK,fnc,value)
    </code>
    """
    def __init__(self,widget=None,value=None,**params):
        params.setdefault('cls','tool')
        button._button.__init__(self,**params)
        if widget:
            self.setwidget(widget)
        self.value = value

    def setwidget(self,w):
        self.widget = w


    def resize(self,width=None,height=None):
        self.widget.rect.w,self.widget.rect.h = self.widget.resize()
        
        return self.widget.rect.w,self.widget.rect.h
 
    def paint(self,s):
        self.widget.paint(surface.subsurface(s,self.widget.rect))


class BoringDialog(gui.table.Table):
    """A boring dialog window.
    """
    def __init__(self,main,**params):
        params.setdefault('cls','dialog')
        gui.table.Table.__init__(self,**params)
                
        self.tr()
        self.td(main,cls=self.cls+".main")

class Countdown(pygame.sprite.Sprite):
    def __init__(self, number=5):
        self.number = number        
        self.digits = int(log10(number))+1

    def display():
        chars = repr((10**self.digits) + self.number)[1:]
        #for c in chars:
            # TODO: finish this
        return

    def update():
        self.number -= 1

def skillButtonImage(skillName, amount):
    x = pygame.image.load(g.RESOURCES_PATH+"pixmaps/"+skillName+".png")
    if amount < 0:
        amount = 0
    elif amount > 99:
        amount = 99

    n = repr(100+amount)[-2:]
    s = stringToBitmap(n)    
    x.blit(s, ( (x.get_width()-s.get_width())/2 ,2))
    return x

highlighted_chars = [int(x[:-4]) for x in os.listdir(g.RESOURCES_PATH+"pixmaps/font-highlight/")]

def stringToBitmap(theString, highlight=True):
    if len(theString)==0:
        theString = " "
    chars = []
    for c in theString:
        if ord(c)>= 32 & ord(c) <= 126:
            if highlight and (ord(c) in highlighted_chars):
                chars.append(g.fontHighlightCache.get(c))
            else:
                chars.append(g.fontCache.get(c))
    totwidth = sum([m.get_size()[0] for m in chars])
    outbuf = pygame.Surface([totwidth,chars[0].get_size()[1] ],SWSURFACE|SRCALPHA|SRCCOLORKEY).convert_alpha()
    outbuf.fill((0,0,0,0))
    temp = 0
    for m in chars:
        outbuf.blit(m,(temp,0))
        temp += m.get_size()[0]
    return outbuf

def createLayout(theString, width, height, align, surfaceOut=True, highlight=True):
    myLay = layout.Layout(pygame.Rect(0,0,width,height))
    myLay.add(align)
    parags = theString.split("\n")
    for i in range(0,len(parags)):
        for word in parags[i].split(" "):
            e = Obj()
            e.image = stringToBitmap(word, highlight=highlight)
            e.rect = pygame.Rect(0,0,e.image.get_width(),e.image.get_height())
            myLay.add(e) #inline object
            myLay.add((6,18)) #space
            myLay.resize()
            
        if not (i == len(parags)-1):
            myLay.add((0,18))

    if surfaceOut:
        text = pygame.Surface([width,height],SWSURFACE|SRCALPHA).convert_alpha()
        text.fill((0,0,0,0))
        for e in myLay.widgets:
            text.blit(e.image,(e.rect.x,e.rect.y))
    else:
        text = myLay

    return text
