"""sprite.py

Part of the Tundra project.
(C) 2006 Rebuild Software
http://rebui.ld.net.au

This file contains the various derivatives of pygame.sprite.Sprite used.
"""

import globals as g

try:
    import pygame
    from pygame.locals import *
except ImportError:
    raise ImportError, "You don't have Pygame installed."
try:
    assert pygame.image.get_extended()
except AssertionError:
    raise ImportError, "Your SDL_image does not support PNG images!"


class InterSprite(pygame.sprite.Sprite):
    def __init__(self, inter, game):
        pygame.sprite.Sprite.__init__(self)
        self.inter = inter
        self.game = game
        self.isMasked = self.inter.drawIfTerrainUnderneath
        self.rect = self.inter.frames[0].get_rect()
        self.rect.x = self.inter.posX
        self.rect.y = self.inter.posY
        self.currentFrame = 0
        self.frameCount = len(inter.frames)
        self.image = self.inter.frames[self.currentFrame]
        if self.isMasked: # needed to prevent unintentional widespread sprite damage
            self.inter.frames = [f.copy() for f in self.inter.frames]
            self.unmaskedFrames = self.frameCount

            smackfu = self.game.foreground.get_rect().clip(self.rect)
            sampler = self.game.foreground.subsurface(smackfu)
            self.mask = pygame.Surface(sampler.get_size(),SWSURFACE|SRCCOLORKEY)

            self.mask.lock()
            sampler.lock()
            for i in range(0,sampler.get_height()):
                for j in range(0,sampler.get_width()):
                    if sampler.get_at((j,i))[3] == 0:
                        self.mask.fill( g.DEFAULT_COLOURKEY, ((j,i),(1,1)))
                    else:
                        self.mask.fill( (0,0,0,255), ((j,i),(1,1)))
            sampler.unlock()
            self.mask.unlock()
            self.mask.set_colorkey((0,0,0,255))
            self.mask = self.mask.convert_alpha()

    def update(self):
        if self.isMasked and self.unmaskedFrames > 0:
            self.inter.frames[self.currentFrame].blit(self.mask, (0,0))
            self.inter.frames[self.currentFrame].set_colorkey(g.DEFAULT_COLOURKEY)
            self.inter.frames[self.currentFrame] = self.inter.frames[self.currentFrame].convert_alpha()            
            self.inter.frames[self.currentFrame].set_colorkey(g.DEFAULT_COLOURKEY2)
            self.inter.frames[self.currentFrame] = self.inter.frames[self.currentFrame].convert_alpha()
            self.image = self.inter.frames[self.currentFrame]            
            self.unmaskedFrames -= 1
        return


class HazardSprite(InterSprite): # despite the name, this is just a perpetually looped animation
    def __init__(self, inter, game):
        InterSprite.__init__(self, inter, game)

    def update(self):
        self.currentFrame += 1;
        self.currentFrame %= self.frameCount
        self.image = self.inter.frames[self.currentFrame]
        InterSprite.update(self)
        return

class TrapSprite(InterSprite): # triggered animation
    def __init__(self, inter, game):
        InterSprite.__init__(self, inter, game)
        self.animating = False

    def trigger(self):
        self.animating = True
        self.update()

    def update(self):
        if self.animating:
            self.currentFrame += 1;
            if (self.currentFrame == self.frameCount):
                self.currentFrame = 0
                self.animating = False
            self.image = self.inter.frames[self.currentFrame]
            InterSprite.update(self)
        return



class EntranceSprite(InterSprite):
    def __init__(self, inter, game):
        InterSprite.__init__(self, inter, game)

    def update(self):
        if self.game.hatchesOpen and (self.currentFrame < self.frameCount-1):
            self.currentFrame += 1
            self.image = self.inter.frames[self.currentFrame]
        InterSprite.update(self)
        return
    
class SpriteGroup(pygame.sprite.OrderedUpdates):
    def __init__(self, *args):
        pygame.sprite.OrderedUpdates.__init__(self, *args)
    
    #change frame of reference of rects
    def drawShift(self, surface, shiftRect):        
        spritedict = self.spritedict
        surface_blit = surface.blit
        dirty = self.lostsprites
        self.lostsprites = []
        dirty_append = dirty.append
        for s in [spr for spr in self.sprites() if spr.rect.colliderect(shiftRect) ]:
            r = spritedict[s]
            newrect = surface_blit(s.image, s.rect.move(-shiftRect.x,-shiftRect.y))
            if r is 0:
                dirty_append(newrect)
            else:
                r = r.move(-shiftRect.x,-shiftRect.y)
                if newrect.colliderect(r):
                    dirty_append(newrect.union(r))
                else:
                    dirty_append(newrect)
                    dirty_append(r)
            spritedict[s] = newrect.move(shiftRect.x,shiftRect.y)
        return dirty
    
    def dirtyRects(self, clippy, shiftRect):
        # did I mention I -love- the complete lack of abstraction
        spritedict = self.spritedict
        dirty = self.lostsprites
        for s in self.sprites():
            r = spritedict[s]
            newrect = s.rect.move(-shiftRect.x,-shiftRect.y).clip(clippy)
            if r is 0:
                dirty.append(newrect)
            else:
                r = r.move(-shiftRect.x,-shiftRect.y)
                if newrect.colliderect(r):
                    dirty.append(newrect.union(r))
                else:
                    dirty.append(newrect)
                    dirty.append(r)
        return dirty
