"""title.py

Part of the Tundra project.
(C) 2006 Rebuild Software
http://rebui.ld.net.au

This file contains the main loop for the title screen.
"""

import globals as g

import pygame
from pygame.locals import *

import os, sys; sys.path.insert(0, "..")

import font, level, dialog, game

from pgu import gui
from pgu.gui.const import *


def renderTitle(colour=(255,255,255,255)):
    surface = pygame.Surface([g.SCREEN_WIDTH, g.SCREEN_HEIGHT],SWSURFACE|SRCALPHA).convert_alpha()
    surface.fill(colour)
    bg = pygame.image.load(g.RESOURCES_PATH+"pixmaps/title_tmp.png")
    r = bg.get_rect()
    r.center = surface.get_rect().center
    surface.blit(bg, r)
    return surface

class Scrolly:
    def __init__(self, scrollString):
        self.xpos = g.SCREEN_WIDTH
        self.index = 0
        self.surfaces = []
        self.pause = True
        self.pauseTimer = 0
        for sentence in scrollString.split("\n"):
            t = font.stringToBitmap(sentence)
            r = t.get_rect()
            self.surfaces.append(pygame.Surface([g.SCREEN_WIDTH, t.get_height()], SWSURFACE|SRCALPHA).convert_alpha())
            self.surfaces[-1].fill((0,0,0,0))
            r.center = self.surfaces[-1].get_rect().center 
            self.surfaces[-1].blit(t, r)

        self.trailingRect = self.surfaces[-1].get_rect()
        self.leadingRect = self.surfaces[0].get_rect()
        self.trailingRect.x = -g.SCREEN_WIDTH
        self.trailingRect.y = self.leadingRect.y = g.TITLE_SCROLL_YPOS
        
    def update(self):
        t = g.clock.get_time()

        if t<0 or t>1000:            # fucking apple
            t = 10
        if self.pause:
            self.pauseTimer += t
            if self.pauseTimer >= g.TITLE_SCROLL_PAUSE:
                self.pauseTimer = 0
                self.index += 1
                self.index %= len(self.surfaces)
                self.pause = False
                self.xpos = g.SCREEN_WIDTH
                self.update()
        else:
            self.xpos -= g.TITLE_SCROLL_SPEED*t/1000.
            if self.xpos < 0.0:
                self.xpos = 0.0
                self.pause = True
            self.leadingRect.x = int(self.xpos)
            self.trailingRect.x = int(self.xpos-g.SCREEN_WIDTH)

    def draw(self, surface):
        surface.blit(self.surfaces[self.index], self.leadingRect)
        surface.blit(self.surfaces[self.index-1], self.trailingRect)

class Title:
    def __init__(self, parent):
        g.mouse.changeCursor("hand")

        self.app = gui.App()
        self.app.connect(gui.QUIT,self.app.quit,None) 
        c = gui.Table(width=570,height=248) # bounding box used for the menu
        c.td(gui.Spacer(64,58), col=0, row=0)
        b = font.Icon(pygame.image.load(g.RESOURCES_PATH+"pixmaps/title_play.png").convert_alpha())
        b.connect(CLICK, parent.switchLoops, "MainLoop")
        c.td(b, col=0, row=1, align=1)
        c.td(gui.Spacer(64,17), col=0, row=2)
        b = font.Icon(pygame.image.load(g.RESOURCES_PATH+"pixmaps/title_options.png").convert_alpha())
        opts = dialog.LoaderDialog(startPage="options")
        opts.parent = parent
        def optionsCallback(): opts.renderPage("options"); opts.open()
        b.connect(CLICK, optionsCallback)
        c.td(b, col=0, row=3, align=1)
        c.td(gui.Spacer(64,75), col=0, row=4)
        b = font.Icon(pygame.image.load(g.RESOURCES_PATH+"pixmaps/title_exit.png").convert_alpha())
        b.connect(CLICK, self.quit)
        c.td(b, col=0, row=5, align=1)

        
        self.app.init(c)
        
        self.splash = renderTitle()
    
        # lets invent some statistics
        numLevels = 0
        unbeatenLevels = 0
        points = 0
        savedLems = 0
        for c in g.levelDB.values():
            for d in c.values():
                for l in d.values():
                    if not l.__class__ == str:
                        numLevels += 1
                        if l.bestTime == 0:
                            unbeatenLevels += 1
                        else:
                            points += l.bestScore
                            savedLems += l.bestSaved
                        
    
        scrollMessage = g.msg("scroll_header")
        scrollMessage += repr(numLevels)+g.msg("scroll_stats1")+repr(unbeatenLevels)+g.msg("scroll_stats2")    
        scrollMessage += g.msg("scroll_stats3") + repr(savedLems) + g.msg("scroll_stats4")
        scrollMessage += g.msg("scroll_stats5") + repr(points) + g.msg("scroll_stats6")
        scrollMessage += g.msg("scroll_notMasterPlayer") + g.msg("scroll_footer") + g.msg("scroll_credits")
        self.scroll = Scrolly(scrollMessage)
        
        #scrollRect = Rect(self.scroll.trailingRect)
        
    def loop(self):
        g.screen.blit(self.splash, (0,0))
        for e in pygame.event.get():
            if (e.type is QUIT) or (e.type is KEYDOWN and e.key == K_ESCAPE): self.quit()
            self.app.event(e)
        self.scroll.update()
        self.scroll.draw(g.screen)   
        self.app.paint(g.screen)
        pygame.display.flip()
#        pygame.display.update(scrollRect)
        g.clock.tick(g.TITLE_FPS)
        
    def quit(self):
        raise SystemExit
